using System;

using EvoLab.CoreApi;
using EvoLab.CoreApi.ModuleInterfaces;
using EvoLab.CoreApi.ModuleTemplates;
using EvoLab.CoreApi.Parameters;

using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Random;

namespace EvoAlg.Ue1
{
    public class Ue1_Population : VectorPopulation<DoubleVector, double>
    {
        public readonly double min;
        public readonly double max;

        public Ue1_Population
            (double min, double max, int vectorLength, int μ, int λ,
            IndividualFactory<EvoLab.CoreApi.Individuals.DoubleVector> allocateNewIndividual)
            : base(vectorLength, μ, λ, allocateNewIndividual)
        {
            this.min = min;
            this.max = max;
        }
    }

    #region Initialization

    [Author("Lieferumfang von EvoLab")]
    [Description("Initialisierungsmodul")]
    [Identifier("Ue1_Initialization")]

    [IntParameter("μ", 30)]
    [IntParameter("λ", 150)]
    
    [DoubleParameter("Bereichs Minimum", -100.0)]
    [DoubleParameter("Bereichs Maximum", 100.0)]

    [IntParameter("Laenge der Individuen", 25)]

    public class Ue1_Initialization : InitializationBase<DoubleVector, Ue1_Population> 
    {
        public override Ue1_Population CreatePopulation(string name, IWorld world, ISimulationControl mySimControl)
        {
            if (world.GetLocalInteger(name, "μ") < 1)
                throw new ArgumentOutOfRangeException (
                    "μ ist kleiner als 1. Keine leere Population möglich.");

            if (world.GetLocalInteger(name, "λ") < 1)
                throw new ArgumentOutOfRangeException (
                    "λ ist kleiner als 0. Keine negativ-anzahlige Nachkommenschaft möglich.");

            if (world.GetLocalInteger(name, "Laenge der Individuen") < 0)
                throw new ArgumentOutOfRangeException (
                    "Laenge der Individuen darf nicht kleiner als 0 sein.");

                        int μ = world.GetLocalInteger(name, "μ");
            int length = world.GetLocalInteger(name, "Laenge der Individuen");
            Ue1_Population population = new Ue1_Population (
                world.GetLocalDouble(name, "Bereichs Minimum"),
                world.GetLocalDouble(name, "Bereichs Maximum"),
                length,
                μ,
                world.GetLocalInteger(name, "λ"),
                (new DoubleVectorFactory (length)));

            foreach (DoubleVector ind in population.Pop) {
                EvoLab.CoreApi.Operators.Initialization.RandomizeDoubleArray (
                    world.RandomNumberGenerator, ind.vector,
                    world.GetLocalDouble(name,"Bereichs Minimum"),
                    world.GetLocalDouble(name, "Bereichs Maximum"));
            }
                        
                        return population;
        }
    }

    #endregion Initialization

    #region Recombination

    [Author("Lieferumfang von EvoLab")]
    [Description("Vektoraddition")]
    [Identifier("Ue1_Rekombination_Linear")]
    public class Ue1_Rekombination_Linear : EARecombination<DoubleVector, Ue1_Population>
    {
        private IRandom rand;
        private double min, max;

        public override void Init(
            string name,
                    Ue1_Population population,
            IWorld world)
        {
            base.Init(name, population, world);

            rand = world.RandomNumberGenerator;

            min = population.min;
            max = population.max;
            if (min > max)
            {
                throw new ArgumentOutOfRangeException("Bereichs Minimum > Bereichs Maximum");
            }

            //2 Eltern, 1 Kind
            InitNumbers(2, 1);
        }

        protected override void Recombine(DoubleVector[] parents, DoubleVector[] children)
        {
            DoubleVector child = children[0];
            DoubleVector parent1 = parents[0];
            DoubleVector parent2 = parents[1];

            double alpha = rand.NextGaussian() + 0.5d;

            for (int i = 0; i < child.vector.Length; ++i)
            {
                child.vector[i] = parent1.vector[i] * alpha + parent2.vector[i] * (1.0d - alpha);
/*                child.vector[i] = (child.vector[i] < min) ? min : child.vector[i];
                child.vector[i] = (child.vector[i] > max) ? max : child.vector[i];*/

            }
        }
    }

    #endregion Recombination

    #region Mutation

    [Author("Lieferumfang von EvoLab")]
    [Description("Mutation")]
    [Identifier("Ue1_Mutation")]

    [IntParameter("Mittlere Abweichung pro Gen in Prozent", 5)]

    public sealed class Ue1_Mutation
    : Mutation <DoubleVector, Ue1_Population>
    {
        private IRandom rand;
        private double deviation;
        private double min, max;

        public override void Init(
            string name,
                    Ue1_Population population,
            IWorld world)
        {
            base.Init (name, population, world);

            rand = world.RandomNumberGenerator;

            deviation = world.GetLocalInteger(name, "Mittlere Abweichung pro Gen in Prozent");
            if (!((deviation < 100) && (deviation > 0)))
                throw new ArgumentOutOfRangeException ("!((deviation <= 100) && (deviation > 0))");

            min = population.min;
            max = population.max;
            if (min > max)
            {
                throw new ArgumentOutOfRangeException("Bereichs Minimum > Bereichs Maximum");
            }
        }

        protected override void Mutate (DoubleVector mutant)
        {
            //TODO: Eine ZZ für alle AlLelen-dingenser, oder für jedes
            //ne eigene ???
            for (int i=0; i<mutant.vector.Length; ++i) {
                mutant.vector[i] = mutant.vector[i] * ( 100.0d - (double)deviation * rand.NextGaussian() ) / 100.0d;
                mutant.vector[i] = (mutant.vector[i]< min) ? min : mutant.vector[i];
                mutant.vector[i] = (mutant.vector[i] > max) ? max : mutant.vector[i];
            }
        }        
    }

    #endregion Mutation

}
