/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using System.Reflection;
using System.Collections;

namespace EvoLab.Common.Com
{
    /// <summary>
    /// In dieser Klasse sind allgemeine (generische) 
    /// Statische Methoden
    /// fr die Serialisierung bzw. Deserialisierung von
    /// Objekten zur bertragung ber den Kommunikationskanal
    /// eingekapselt. 
    /// </summary>
    public class ComWrapper
    {
        /// <summary>
        /// Konstuktor, wird nicht bentigt. 
        /// </summary>
        private ComWrapper() { /* nur statische Methoden, also keine Instanzen dieses Objektes */ }

        /// <summary>
        /// Der Serializer ist eine Klassenvariable, 
        /// damit er nicht immer neu instantiiert werden muss. 
        /// Abbildung (Type->XmlSerializer)
        /// </summary>
        private static Hashtable serializers = new Hashtable();
        /// <summary>
        /// Die Liste ist static, damit sie nicht stndig neu gefllt wird. 
        /// </summary>
        private static List<Type> extraTypes = null;

        /// <summary>
        /// Serialisiert ein angegebenenes Objekt mit
        /// dem angegebenen Typ fr die Kommunikation
        /// </summary>
        /// <param name="o">zu serialisierendes Objekt</param>
        /// <param name="t">der Typ von Objekt o</param>
        /// <returns>XML-Serialisierter UTF-16 String</returns>
        /// <exception cref="Exception">Wenn beim serialisieren ein belieber Fehler auftritt,
        /// wird sofort eine Exception geworfen</exception>
        /// <remarks>
        ///  Geerbte Typen, die in dieser Klasse benutzt werden, mssen im 
        /// Klassen-Header mit folgender Anweisung deklariert werden:
        /// [XmlInclude(typeof(EvoLab.Common.ModuleInfo.FitnessModuleInfo))]
        /// </remarks>
        public static string ComSerialize(Object o, Type t)
        {
            StringWriter sw = new StringWriter ();
            ComSerialize (o, t, sw);
            return sw.ToString ();
        }
        
        /// <summary>
        /// Serialisiert ein angegebenenes Objekt mit
        /// dem angegebenen Typ fr die Kommunikation
        /// </summary>
        /// <param name="o">zu serialisierendes Objekt</param>
        /// <param name="t">der Typ von Objekt o</param>
        /// <returns>Serialisierter String</returns>
        /// <exception cref="Exception">Wenn beim serialisieren ein belieber Fehler auftritt,
        /// wird sofort eine Exception geworfen</exception>
        /// <remarks>
        ///  Geerbte Typen, die in dieser Klasse benutzt werden, mssen im 
        /// Klassen-Header mit folgender Anweisung deklariert werden:
        /// [XmlInclude(typeof(EvoLab.Common.ModuleInfo.FitnessModuleInfo))]
        /// </remarks>
        public static void ComSerialize(Object o, Type t, TextWriter sw)
        {
            LoadTypesFromCommon();
            // schauen, ob der serializer fr den typ schon bekannt ist...
            XmlSerializer serializer;
            if (serializers.ContainsKey(t))
            {
                serializer = (XmlSerializer)serializers[t];
            }
            else
            {
                // wenn unbekannt: neu instantiieren und in die Liste dazu...
                serializer = new XmlSerializer(t, extraTypes.ToArray());
                serializers.Add(t, serializer);
            }

            try
            {
                serializer.Serialize(sw, o);
            }
            catch (Exception ex)
            {
                throw new Exception("ComSerialize: Fehler bei Typ \"" + t.ToString() + "\"", ex);
            }
        }


        /// <summary>
        /// De-Serialisiert ein angegebenenes Objekt mit
        /// dem angegebenen Typ fr die Kommunikation
        /// </summary>
        /// <param name="str">String, aus dem Wieder ein Objekt gebaut werden soll</param>
        /// <param name="t">der Typ des zu erstellenden Objektes</param>
        /// <returns>Eine neue Instanz vom Typ t mit dem Inhalt von str</returns>
        /// <exception cref="Exception">Wenn beim deserialisieren ein belieber Fehler auftritt,
        /// wird sofort eine Exception geworfen</exception>
        public static Object ComDeserialize(string str, Type t)
        {
            LoadTypesFromCommon();
            // schauen, ob der serializer fr den typ schon bekannt ist...
            XmlSerializer deserializer;
            if (serializers.ContainsKey(t))
            {
                deserializer = (XmlSerializer)serializers[t];
            }
            else
            {
                // wenn unbekannt: neu instantiieren und in die Liste dazu...
                deserializer = new XmlSerializer(t, extraTypes.ToArray());
                serializers.Add(t, deserializer);
            }
            Object res = null; 
            try
            {
                StringReader reader = new StringReader(str);
                res = deserializer.Deserialize(reader);
            }
            catch (Exception ex)
            {

                throw new Exception("ComDeSerialize: Fehler bei Typ \"" + t.ToString() + "\"" +  ex.Message + ex.StackTrace);

            }
            return res;
        }



        /// <summary>
        /// Ldt alle Typen in der evocommon.dll
        /// in die statische Liste dieser Klasse, 
        /// wenn die Liste noch nicht initialisiert wurde. 
        /// Dies hat den Vorteil, dass sie nur einmal initialisiert werden muss, 
        /// was Zeit bei der bertratung durch den Kommunikationsteil spart. 
        /// 
        /// Es werden alle Klassen aus evcommon geladen, also je weniger Klassen
        /// in dieser DLL unntigerweise geladen werden, desto schnell geht das hier!!!
        /// </summary>
        private static void LoadTypesFromCommon()
        {
            //Dies ist ein Workaround, da beliebige geerbte Klassen enthalten sein knnen. 
            //Diese Typen mssen jedoch in EvoLab.Common sein, 
            //da sie sonst eh nicht auf Clientseite verfgbar sind. 
            //Wir fgen sie einfach mal dazu, dass knnte vielleicht langsam sein!
            if (extraTypes == null)
            {
                extraTypes = new List<Type>();
                foreach (Type et in typeof(ComWrapper).Assembly.GetTypes())
                {
                    bool bHasEmptyConstructor = false;
                    foreach (ConstructorInfo con in et.GetConstructors())
                    {
                        bHasEmptyConstructor = bHasEmptyConstructor || (con.GetParameters().Length == 0);
                    }
                    if (!et.IsAbstract && !et.IsInterface && bHasEmptyConstructor && et.IsPublic &&
                        !et.ContainsGenericParameters)
                    {
                        extraTypes.Add(et);
                    }
                }
            }
        }
    }
}
