/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.Runtime.Serialization;


namespace EvoLab.Common
{


    /// <summary>
    /// Diese Exception wird vom Core geworfen, wenn eine Aktion für
    /// eine Simulations-Id durchgeführt werden soll, die im
    /// Core nicht bekannt ist.
    /// </summary>
    /// @cond
    /// <author>Daniel Hofmann</author>
    /// <version>$Revision: 2625 $</version>
    /// <id>$Id: EvoLabException.cs 2625 2008-04-25 10:16:16Z hofmandl $</id>
    /// @endcond
    [Serializable]
    public class UnknownIdException : EvoLabException
    {

        public readonly string id;

        //Die folgenden beiden Konstruktoren sollten durch einen
        //ersetzt werden, der nur noch die id aufnimmt, sobald das
        //id-Feld mit über die Com übertragen wird.
        public UnknownIdException(string message)
            : base(message)
        {
        }

        public UnknownIdException(string message, string id)
            : base(message + ": " + id)
        {
            this.id = id;
        }

        public override void GetObjectData(SerializationInfo si, StreamingContext context)
        {
            base.GetObjectData (si, context);
            si.AddValue("id", id, typeof(string));
        }

        protected UnknownIdException(SerializationInfo si, StreamingContext context)
            : base(si, context)
        { 
            this.id = si.GetString("id");
        }
    }

    /// <summary>
    /// Diese Exception wird vom Core geworfen, wenn eine
    /// statusverändernde Operation auf eine Simulation ausgeführt
    /// werden sollte, diese den Status aber gar nicht verändert hat.
    /// </summary>
    /// @cond
    /// <author>Daniel Hofmann</author>
    /// <version>$Revision: 2625 $</version>
    /// <id>$Id: EvoLabException.cs 2625 2008-04-25 10:16:16Z hofmandl $</id>
    /// @endcond
    [Serializable]
    public class OperationException : EvoLabException
    {
        public readonly string operation;
        public readonly SimulationStatus actualStatus;
        
        public OperationException(string operation, SimulationStatus actualStatus)
            : base(String.Format("Operation '{0}' kann nicht im Zustad '{1}' ausgeführt werden."
                ,operation, actualStatus))
        {
            this.actualStatus = actualStatus;
            this.operation = operation;
        }

        //Behelfskonstruktor, bis die Felder actualStatus und
        //requiredStatus evtl. mit der Com übertragen werden.
        public OperationException(string message)
            : base(message)
        {
        }

        public override void GetObjectData(SerializationInfo si, StreamingContext context)
        {
            base.GetObjectData (si, context);
            si.AddValue("actualStatus", actualStatus, typeof(SimulationStatus));
            si.AddValue("operation", operation, typeof(string));
        }

        protected OperationException(SerializationInfo si, StreamingContext context)
            : base(si, context)
        { 
            actualStatus = (SimulationStatus)si.GetValue("actualStatus", typeof (SimulationStatus));
            operation = si.GetString("operation");
        }
    }

    /// <summary>
    /// Wird geworfen, wenn der Benutzer nicht die nötigen Rechte hat,
    /// eine Aktion durchzuführen.
    /// </summary>
    /// @cond
    /// <author>Daniel Hofmann</author>
    /// <version>$Revision: 2625 $</version>
    /// <id>$Id: EvoLabException.cs 2625 2008-04-25 10:16:16Z hofmandl $</id>
    /// @endcond
    [Serializable]
    public class PermissionException : EvoLabException
    {

        public readonly string userId;
        public readonly string objectId;
        public readonly Rights missingRight;

        /// <summary>
        /// Erzeugt eine Nachricht und setzt userId und MissingRight. Dieser
        /// Konstruktor soll verwenet werden wenn ein Benutzer ein Recht Fehlt
        /// um eine Aktion durchzuführen.
        /// </summary>
        /// <param name="right"></param>
        /// <param name="userId"></param>
        public PermissionException(Rights right, string userId)
            : base(String.Format("Benutzer '{0}' wurde der Zugriff verweigert: Fehlendes Recht '{1}'", 
            userId, right))
        {
            this.userId = userId;
            this.missingRight = right;
        }

        /// <summary>
        /// Erzeugt eine Nachricht und setzt userId und ObjectId. Dieser Konstruktor soll verwendet
        /// werden wenn der Zugriff auf ein Objekt verweigert wurde.
        /// </summary>
        /// <param name="message"></param>
        /// <param name="userId"></param>
        public PermissionException(string userId, string objectId)
            : base(String.Format("Benutzer '{0}' wurde der Zugriff auf Objekt '{1}' verweigert.",
            userId, objectId))
        {
            this.missingRight = Rights.UNDEFINED;
            this.objectId = objectId;
            this.userId = userId;
        }

        /// <summary>
        /// Standard-Konstruktor.
        /// </summary>
        /// <param name="msg"></param>
        public PermissionException(string msg)
            : base(msg)
        {

        }

        public override void GetObjectData(SerializationInfo si, StreamingContext context)
        {
            base.GetObjectData (si, context);
            si.AddValue("userId", userId, typeof(string));
            si.AddValue("missingRight", missingRight, typeof(Rights));
            si.AddValue("objectId", objectId, typeof(string));
        }

        /// <summary>
        /// Konstruktor zum deserialisieren.
        /// </summary>
        /// <param name="si"></param>
        /// <param name="context"></param>
        protected PermissionException(SerializationInfo si, StreamingContext context)
            : base(si, context) 
        {
            missingRight = (Rights)si.GetValue("missingRight", typeof(Rights));
            userId = si.GetString("userId");
            objectId = si.GetString("objectId");
        }

    }
    

    /// <summary>
    /// Diese Exception dient zur Klassifizierung der Exceptions, die
    /// vom EvoCore aus nach außen gereicht werden.
    ///
    /// Wird vom Core diese Exception geworfen, so deutet dies in der
    /// Regel auf einen coreinternen Fehler hin. Alle Klassen, die von
    /// dieser Exception erben wiederum deuten auf unkritische Fehler hin.
    /// </summary>
    /// @cond
    /// <author>Daniel Hofmann</author>
    /// <version>$Revision: 2625 $</version>
    /// <id>$Id: EvoLabException.cs 2625 2008-04-25 10:16:16Z hofmandl $</id>
    /// @endcond
    [Serializable]
    public class EvoLabException : ApplicationException
    {

        public EvoLabException(string message)
            : base(message)
        {
        }

        protected EvoLabException(SerializationInfo si, StreamingContext context)
            : base(si, context) { }

    }
}
