/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.IO;
using System.Text;

namespace EvoLab.Common {

    /// <summary>
    /// Klasse, die beim Vereinheitlichen der Lognachrichten eines Programms hilft.
    ///
    /// Wichtig: Diese Klasse ist nicht statisch, muss also, wenn dieselbe Instanz
    /// an mehreren Stellen verwendet werden soll, an diese Stellen weitergegeben werden.
    /// </summary>
    ///
    /// Die Instanzmember dieser Klasse sind threadsicher. 
    /// @cond
    /// <author>Daniel Hofmann</author>
    /// <version>$Revision: 2625 $</version>
    /// <id>$Id: Log.cs 2625 2008-04-25 10:16:16Z hofmandl $</id>
    /// @endcond
    public class Logger : MarshalByRefObject {

        private TextWriter outputTarget;
        private string commonPrefix;

        /// <summary>
        /// Erstellt eine neue Logger-Instanz, die auf die Konsole
        /// ausgibt und kein Prefix hat.
        /// </summary>
        public Logger () : this (Console.Out, null) {
        }
        
        /// <summary>
        /// Erstellt eine neue Logger-Instanz, die auf die Konsole
        /// ausgibt.
        /// </summary>
        /// <param name="commonPrefix">Prefix, das allen Nachrichten,
        /// die geloggt werden vorangestellt wird.</param>
        public Logger (string commonPrefix) : this (Console.Out, commonPrefix) {
        }

        /// <summary>
        /// Erstellt eine neue Logger-Instanz ohne Prefix.
        /// </summary>
        /// <param name="outputTarget">TextWriter, in den geloggt wird.</param>
        /// <exception cref="ArgumentNullException">falls outputTarget null ist.</exception>
        public Logger (TextWriter outputTarget) : this (outputTarget, "") {
        }
        
        /// <summary>
        /// Erstellt eine neue Logger-Instanz.
        /// </summary>
        /// <param name="outputTarget">TextWriter, in den geloggt wird.</param>
        /// <param name="commonPrefix">Prefix, das allen Nachrichten,
        /// die geloggt werden vorangestellt wird.</param>
        /// <exception cref="ArgumentNullException">falls outputTarget null ist.</exception>
        public Logger (TextWriter outputTarget, string commonPrefix) {
            if (outputTarget == null)
                throw new ArgumentNullException ("outputTarget ist null");
            this.outputTarget = outputTarget;
            this.commonPrefix = commonPrefix;
        }

        /// <summary>
        /// Setzt den Ausgabe-TextWriter oder ruft ihn ab.
        /// </summary>
        public TextWriter Out {
            get {
                return outputTarget;
            }
            set {
                if (value != outputTarget) {
                    lock (this) {
                        outputTarget = value;
                    }
                }
            }
        }

        /// <summary>
        /// Setzt das Log-Präfix oder ruft es ab.
        /// </summary>
        public string CommonPrefix {
            get {
                return commonPrefix;
            }
            set {
                commonPrefix = value;
            }
        }
        
        /// <summary>
        /// Loggt eine formatierte Nachricht mit LogLevel DEBUG.
        /// </summary>
        /// <param name="message">FormatierString.</param>
        /// <param name="arg">Argumente für den Formatierstring.</param>
        /// <exception cref="ArgumentNullException">falls eines der
        /// verwendeten Formatierargumente null ist.</exception>
        /// <exception cref="FormatException">format ist ungültig oder
        /// die Formatierargumente und die übergebenen Objekte passen
        /// nicht zusammen.</exception>
        public void Log (string message, params object[] arg) {
            Log (LogLevel.DEBUG, message, arg);
        }
        
        /// <summary>
        /// Loggt eine formatierte Nachricht.
        /// </summary>
        /// <param name="level">Der LogLevel.</param>
        /// <param name="message">FormatierString.</param>
        /// <param name="arg">Argumente für den Formatierstring.</param>
        /// <exception cref="ArgumentNullException">falls eines der
        /// verwendeten Formatierargumente null ist.</exception>
        /// <exception cref="FormatException">format ist ungültig oder
        /// die Formatierargumente und die übergebenen Objekte passen
        /// nicht zusammen.</exception>
        public void Log (LogLevel level, string message, params object[] arg)
        {
            lock (this)
            {
                if (outputTarget != null)
                {    
                    StringBuilder sb = new StringBuilder ();
            
                    sb.Append (DateTime.Now.ToString ());
                    sb.Append (' ');
                    sb.Append ('[');
                    sb.Append (level.ToString ());
                    sb.Append (']');
                    sb.Append (' ');

                    if (!String.IsNullOrEmpty(commonPrefix)) {
                        sb.Append (commonPrefix);
                        sb.Append (' ');
                    }
    
                    sb.Append (String.Format(message, arg));

                    outputTarget.WriteLine (sb.ToString());
                    outputTarget.Flush ();
                }
            }
        }

        /// <summary>
        /// Gibt den zugrundeliegenden TextWriter frei.
        /// </summary>
        public void Close () {
            lock (this) {
                if (outputTarget != null) {
                    outputTarget.Close ();
                    outputTarget = null;
                }
            }
        }

        /// <summary>
        /// Überschrieben. Führt zu einer unendlichen Lebendsdauer des Log, indem
        /// das Erstellen eines Lease verhindert wird.
        /// </summary>
        /// <returns>null</returns>
        public override object InitializeLifetimeService()
        {
            return null;
        }

    }

    public enum LogLevel {
        CRITICAL,
        INFO,
        DEBUG
    }

}
