/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using EvoLab.Common.ModuleInfo;
using System.Collections.Generic;
using System.Xml;

namespace EvoLab.Common
{

    /// <summary>
    /// Enthält alle zur Ausführung einer Simulation notwendigen Daten.
    ///
    /// Dieser Typ ist quasi das Rechnerpendant zur XML-Konfigurationsdatei.
    /// </summary>
    /// @cond
    /// <author>Daniel Hofmann</author>
    /// <version>$Revision: 2625 $</version>
    /// <id>$Id: SimConfig.cs 2625 2008-04-25 10:16:16Z hofmandl $</id>
    /// @endcond
    [Serializable]
    public sealed class SimConfig
    {

        //Beschreibung der Simulation
        public string description;

        //Das Initialisierungsmodul für diese Simulation.
        public EvoModuleInfo initializationModule;

        //Das Simulationsmodul für diese Simulation.
        public EvoModuleInfo simulationModule;

        //Liste der sonstigen ModuleInfo-Objekte, die in der
        //Simulation verwendet werden sollen.
        public EvoModuleInfo[] moduleList;

        //Die Liste der Modulidentifier, die vor dem Zyklus ausgeführt
        //werden sollen
        public string[] executeBeforeCycle;

        //Von den Modulen geforderte globale Parameter
        public SerializableDictionary<string, bool> globalBooleanParameters;
        public SerializableDictionary<string, double> globalDoubleParameters;
        public SerializableDictionary<string, int> globalIntParameters;
        public SerializableDictionary<string, int[]> globalIntSequenceParameters;
        public SerializableDictionary<string, string[]> globalStringSequenceParameters;
        public SerializableDictionary<string, string> globalStringParameters;
        public SerializableDictionary<string, string[]> globalOperatorParameters;
        public SerializableDictionary<string, string> globalFileParameters;

        // 14.06.07 MF:
        // @Dani: hab die Clone()-Methode zu deinem Code hinzugefügt (ich hoffe das stört dich nicht)
        // Vielleicht fällt die auch was besseres ein, um die Modulliste zu klonen...

        // Jupp, kein Problem, hab noch den Nullfall berücksichtigt
        // und die neuen Felder zum Klonen hinzugefügt, und die
        // CloneModuleList-Methode direkt in ein neues Array klonen lassen.

        /// <summary>
        /// Diese Methode klont die Modulliste dieses Objekts.
        /// </summary>
        /// <param name="orig">die zu klonende Original-Modulliste</param>
        /// <returns>die neue geklonte Modulliste, oder null, falls
        /// orig auch null ist.</returns>
        private EvoModuleInfo[] CloneModuleList(EvoModuleInfo[] orig)
        {
            if (orig == null)
                return null;

            int i = 0;
            EvoModuleInfo[] newArray = new EvoModuleInfo[orig.Length];
            foreach (EvoModuleInfo pmi in orig)
            {
                newArray[i] = pmi.Clone();
                ++i;
            }
            return newArray;
        }

        /// <summary>
        /// Diese Methode klont dieses Objekt und gibt die neue Instanz zurück.
        /// 
        /// CF: 24.06.07: Try/Catches eingebaut, da hier eine Nullpointerexception liegt. 
        /// </summary>
        /// <returns>das geklonte neue Objekt</returns>
        public SimConfig Clone()
        {
            SimConfig nc = new SimConfig();

            if (this.simulationModule == null) nc.simulationModule = null;
            else nc.simulationModule = this.simulationModule.Clone();
            
            if (this.initializationModule == null) nc.initializationModule = null;
            else nc.initializationModule = this.initializationModule.Clone();
            
            nc.description = this.description;
            
            if (this.moduleList == null) nc.moduleList = null;
            else nc.moduleList = this.CloneModuleList(this.moduleList);
            
            if (this.executeBeforeCycle == null) nc.executeBeforeCycle = null;
            else nc.executeBeforeCycle = (string[]) this.executeBeforeCycle.Clone ();

            nc.globalBooleanParameters = SimConfig.CloneParams<bool>(this.globalBooleanParameters);
            nc.globalDoubleParameters = SimConfig.CloneParams<double>(this.globalDoubleParameters);
            nc.globalIntParameters = SimConfig.CloneParams<int>(this.globalIntParameters);

            nc.globalFileParameters = SimConfig.CloneStringParams(this.globalFileParameters);
            nc.globalStringParameters = SimConfig.CloneStringParams(this.globalStringParameters);

            nc.globalIntSequenceParameters = SimConfig.CloneArrayParams<int>(this.globalIntSequenceParameters);

            nc.globalStringSequenceParameters = SimConfig.CloneStringArrayParams(this.globalStringSequenceParameters);
            nc.globalOperatorParameters = SimConfig.CloneStringArrayParams(this.globalOperatorParameters);

            return nc;
        }

        /// <summary>
        /// Diese Methode klont das übergebene Dictionary.
        /// Der ValueType des Dictionaries muss ein Werttyp sein.
        ///
        /// </summary>
        /// <param name="piArray">die zu klonenden Parameter</param>
        /// <returns>geklonte Parameter und null, falls das gegebene
        /// Dictionary auch null ist.</returns>
        public static SerializableDictionary<string, T> CloneParams<T>
            (SerializableDictionary<string, T> piDict)
            where T : struct
        {
            if (piDict == null)
                return null;
            
            SerializableDictionary<string, T> newPiDict =
                new SerializableDictionary<string, T>();
            
            foreach (string s in piDict.Keys)
            {
                newPiDict.Add(s, piDict[s]);
            }
            return newPiDict;
        }

        /// <summary>
        /// Diese Methode klont das übergebene Dictionary.
        /// Der ValueType des Dictionaries muss string sein.
        ///
        /// </summary>
        /// <param name="piArray">die zu klonenden Parameter</param>
        /// <returns>geklonte Parameter und null, falls das gegebene
        /// Dictionary auch null ist.</returns>
        public static SerializableDictionary<string, string> CloneStringParams
            (SerializableDictionary<string, string> piDict)
        {
            if (piDict == null)
                return null;
            
            SerializableDictionary<string, string> newPiDict =
                new SerializableDictionary<string, string>();
            
            foreach (string s in piDict.Keys)
            {
                newPiDict.Add(s, piDict[s]);
            }
            return newPiDict;
        }

        /// <summary>
        /// Diese Methode klont das übergebene Dictionary.
        /// Der ValueType des Dictionaries muss ein Array eines Werttyps sein.
        /// 
        /// </summary>
        /// <param name="piArray">die zu klonenden Parameter</param>
        /// <returns>geklonte Parameter und null, falls das gegebene
        /// Dictionary auch null ist.</returns>
        public static SerializableDictionary<string, T[]> CloneArrayParams<T>
            (SerializableDictionary<string, T[]> piDict)
            where T : struct
        {
            if (piDict == null)
                return null;
            
            SerializableDictionary<string, T[]> newPiDict =
                new SerializableDictionary<string, T[]>();
            
            foreach (string s in piDict.Keys)
            {
                newPiDict.Add(s, (T[])piDict[s].Clone ());
            }
            return newPiDict;
        }

        /// <summary>
        /// Diese Methode klont das übergebene Dictionary.
        /// Der ValueType des Dictionaries muss ein StringArray sein.
        /// 
        /// </summary>
        /// <param name="piArray">die zu klonenden Parameter</param>
        /// <returns>geklonte Parameter und null, falls das gegebene
        /// Dictionary auch null ist.</returns>
        public static SerializableDictionary<string, string[]> CloneStringArrayParams
            (SerializableDictionary<string, string[]> piDict)
        {
            if (piDict == null)
                return null;
            
            SerializableDictionary<string, string[]> newPiDict =
                new SerializableDictionary<string, string[]>();
            
            foreach (string s in piDict.Keys)
            {
                newPiDict.Add(s, (string[])piDict[s].Clone ());
            }
            return newPiDict;
        }

        //TODO: Das executeBeforeCycle mitspeichern

        /// <summary>
        /// WB: Schreibt owner, description und customID einer SimConfig in 
        /// einen XmlTextWriter
        /// </summary>
        ///
        /// <remarks>
        /// Ignoriert die EvoModuleInfo Liste, die in einer SimConfig
        /// gespeichert ist.
        /// </remarks>
        public void WriteToXML(XmlTextWriter w)
        {

            w.WriteStartElement("SIMCONFIG");

            // Die Beschreibung
            if (!String.IsNullOrEmpty(this.description))
            {
                w.WriteStartElement("DESCRIPTION");
                w.WriteString(this.description);
                w.WriteFullEndElement();
            }

            // owner
            //if (!String.IsNullOrEmpty(this.owner))
            //{
            //    w.WriteAttributeString("owner", this.owner);
            //}
            //else
            //{
            //    w.WriteAttributeString("owner", "Unbekannter Besitzer");
            //}

            // customid
            //if (!String.IsNullOrEmpty(this.customId))
            //{
            //    w.WriteAttributeString("customid", this.customId);
            //}
            //else
            //{
            //    w.WriteAttributeString("customid", "");
            //}

            // Initialisierungsmodul
            if (this.initializationModule != null)
            {
                this.WriteEvoModuleInfo(w, this.initializationModule, "INITIALIZATIONMODULE");
            }

            // Simulationsmodul
            if (this.simulationModule != null)
            {
                this.WriteEvoModuleInfo(w, this.simulationModule, "SIMULATIONMODULE");
            }

            // Liste aller verwendeter normaler Module
            if (this.moduleList != null)
            {
                foreach (EvoModuleInfo emi in this.moduleList)
                {
                    this.WriteEvoModuleInfo(w, emi, "MODULE");
                }
            }
            
            //Die Liste der Modulidentifier, die vor dem Zyklus ausgeführt
            //werden sollen. public string[] executeBeforeCycle;
            if (this.executeBeforeCycle != null)
            {
                if (this.executeBeforeCycle.Length > 0)
                {
                    w.WriteStartElement("EXECUTEBEFORECYCLE");
                    foreach (string s in this.executeBeforeCycle)
                    {
                        if (!String.IsNullOrEmpty(s)) 
                        {
                            w.WriteStartElement("EXECUTEBEFOREITEM");
                            w.WriteAttributeString("name", s);
                            w.WriteFullEndElement();
                        }
                    }
                    w.WriteFullEndElement();
                }
            }

            // Von den Modulen geforderte globale Parameter
            // public SerializableDictionary<string, bool> globalBooleanParameters;
            if (this.globalBooleanParameters != null)
            {
                if (this.globalBooleanParameters.Count != 0)
                {
                    w.WriteStartElement("GLOBALBOOLEANPARAMETERS");
                    this.globalBooleanParameters.WriteXml(w);
                    w.WriteEndElement();
                }
            }
            if (this.globalDoubleParameters != null)
            {
                if (this.globalDoubleParameters.Count != 0)
                {
                    // public SerializableDictionary<string, double> globalDoubleParameters;
                    w.WriteStartElement("GLOBALDOUBLEPARAMETERS");
                    this.globalDoubleParameters.WriteXml(w);
                    w.WriteEndElement();
                }
            }
            if (this.globalIntParameters != null)
            {
                if (this.globalIntParameters.Count != 0)
                {
                    // public SerializableDictionary<string, int> globalIntParameters;
                    w.WriteStartElement("GLOBALINTPARAMETERS");
                    this.globalIntParameters.WriteXml(w);
                    w.WriteEndElement();
                }
            }
            if (this.globalIntSequenceParameters != null)
            {
                if (this.globalIntSequenceParameters.Count != 0)
                {
                    // public SerializableDictionary<string, int[]> globalIntSequenceParameters;
                    w.WriteStartElement("GLOBALINTSEQUENCEPARAMETERS");
                    this.globalIntSequenceParameters.WriteXml(w);
                    w.WriteEndElement();
                }
            }
            if (this.globalStringSequenceParameters != null)
            {
                if (this.globalStringSequenceParameters.Count != 0)
                {
                    // public SerializableDictionary<string, int[]> globalIntSequenceParameters;
                    w.WriteStartElement("GLOBALSTRINGSEQUENCEPARAMETERS");
                    this.globalStringSequenceParameters.WriteXml(w);
                    w.WriteEndElement();
                }
            }
            if (this.globalStringParameters != null)
            {
                if (this.globalStringParameters.Count != 0)
                {
                    // public SerializableDictionary<string, string> globalStringParameters;
                    w.WriteStartElement("GLOBALSTRINGPARAMETERS");
                    this.globalStringParameters.WriteXml(w);
                    w.WriteEndElement();
                }
            }
            if (this.globalOperatorParameters != null)
            {
                if (this.globalOperatorParameters.Count != 0)
                {
                    // public SerializableDictionary<string, string[]> globalOperatorParameters;
                    w.WriteStartElement("GLOBALOPERATORPARAMETERS");
                    this.globalOperatorParameters.WriteXml(w);
                    w.WriteEndElement();
                }
            }
            if (this.globalFileParameters != null)
            {
                if (this.globalFileParameters.Count != 0)
                {
                    // public SerializableDictionary<string, string[]> globalFileParameters;
                    w.WriteStartElement("GLOBALFILEPARAMETERS");
                    this.globalFileParameters.WriteXml(w);
                    w.WriteEndElement();
                }
            }

            w.WriteFullEndElement();
        }

        /// <summary>
        /// Schreibt ein EvoModuleInfo Objekt in eine XMLDatei
        /// </summary>
        private void WriteEvoModuleInfo(XmlTextWriter w, EvoModuleInfo emi, string elementText)
        {   
            w.WriteStartElement(elementText);

            w.WriteAttributeString("assemblyName", emi.assemblyName);
            w.WriteAttributeString("className", emi.className);
            w.WriteAttributeString("moduleType", emi.moduleType.ToString());
            w.WriteAttributeString("identifier", emi.identifier);

            if (emi.booleanParameters != null)
            {
                if (emi.booleanParameters.Count != 0)
                {
                    w.WriteStartElement("BOOLEANPARAMETERS");
                    emi.booleanParameters.WriteXml(w);
                    w.WriteEndElement();
                }
            }

            if (emi.doubleParameters != null)
            {
                if (emi.doubleParameters.Count != 0)
                {
                    w.WriteStartElement("DOUBLEPARAMETERS");
                    emi.doubleParameters.WriteXml(w);
                    w.WriteEndElement();
                }
            }

            if (emi.intParameters != null)
            {
                if (emi.intParameters.Count != 0)
                {
                    w.WriteStartElement("INTPARAMETERS");
                    emi.intParameters.WriteXml(w);
                    w.WriteEndElement();
                }
            }

            if (emi.intSequenceParameters != null)
            {
                if (emi.intSequenceParameters.Count != 0)
                {
                    w.WriteStartElement("INTSEQUENCEPARAMETERS");
                    emi.intSequenceParameters.WriteXml(w);
                    w.WriteEndElement();
                }
            }

            if (emi.stringSequenceParameters != null)
            {
                if (emi.stringSequenceParameters.Count != 0)
                {
                    w.WriteStartElement("STRINGSEQUENCEPARAMETERS");
                    emi.stringSequenceParameters.WriteXml(w);
                    w.WriteEndElement();
                }
            }

            if (emi.stringParameters != null)
            {
                if (emi.stringParameters.Count != 0)
                {
                    w.WriteStartElement("STRINGPARAMETERS");
                    emi.stringParameters.WriteXml(w);
                    w.WriteEndElement();
                }
            }

            if (emi.operatorParameters != null)
            {
                if (emi.operatorParameters.Count != 0)
                {
                    w.WriteStartElement("OPERATORPARAMETERS");
                    emi.operatorParameters.WriteXml(w);
                    w.WriteEndElement();
                }
            }

            if (emi.fileParameters != null)
            {
                if (emi.fileParameters.Count != 0)
                {
                    w.WriteStartElement("FILEPARAMETERS");
                    emi.fileParameters.WriteXml(w);
                    w.WriteEndElement();
                }
            }

            // Wir benötigen das EndElement eines leeren Elements, welches
            // normalerweiße nach Aufruf von WriteEndElement() durch ein
            // einzeiliges Element dargestellt wird, da ohne Endelement
            // unser reader nicht funktioniert.
            // WriteFullEndElement() schreibt auch bei leeren Elementen
            // das EndElement.
            w.WriteFullEndElement();
        }
        
        public void DumpToConsole() {
            Console.WriteLine("****************** [SIMCONFIG] **********************");
            Console.WriteLine("DESCRIPTION: " + this.description);
            Console.WriteLine("*****************************************************");
        }

        public static SimConfig CreateDummySimConfig ()
        {
            SimConfig simConfig = new SimConfig ();

            simConfig.description = "Hier kann eine Beschreibung der Konfiguration eingefügt werden.";
            simConfig.initializationModule = CreateDummyEvoModuleInfo (ModuleType.INITIALIZATION);
            simConfig.simulationModule = CreateDummyEvoModuleInfo (ModuleType.SIMULATION);
            simConfig.moduleList = new EvoModuleInfo[] { CreateDummyEvoModuleInfo (ModuleType.FITNESS) };
            simConfig.executeBeforeCycle = new string[] { "MY_FITNESS_MODULE_ID" };
            simConfig.globalBooleanParameters = CreateDummyParameters<bool> ("GlobalBooleanParameter", 1, true);
            simConfig.globalDoubleParameters = CreateDummyParameters<double> ("GlobalDoubleParameter", 1, 2.7182818284590452354);
            simConfig.globalIntParameters = CreateDummyParameters<int> ("GlobalIntegerParameter", 1, 23);
            simConfig.globalIntSequenceParameters = CreateDummyParameters<int[]> ("GlobalIntSequenceParameter", 1,
                new int[] {5, 4, 3, 2, 1});
            simConfig.globalStringSequenceParameters = CreateDummyParameters<string[]> ("GlobalStringSequenceParameter", 1,
                new string[] {"a", "b", "c", "d", "e"});
            simConfig.globalStringParameters = CreateDummyParameters<string> ("GlobalStringParameter", 1,
                "So long and thanks for all the fish.");
            simConfig.globalOperatorParameters = CreateDummyParameters<string[]> ("GlobalOperatorParameter", 1,
                new string [] {"Tim", "Karl", "Klößchen", "Gabi"});
            simConfig.globalFileParameters = CreateDummyParameters<string> ("GlobalFileParameter", 1, "passwords.txt");

            return simConfig;
        }

        public static EvoModuleInfo CreateDummyEvoModuleInfo (ModuleType moduleType)
        {
            EvoModuleInfo evoModuleInfo = new EvoModuleInfo ();

            evoModuleInfo.assemblyName = "Der Name der Dll-Datei, in der das Modul hochgeladen wurde.";
            evoModuleInfo.className = "Der Klassenname des Moduls, ggf. mit Namespace-Präfix.";

            evoModuleInfo.moduleType = moduleType;

            evoModuleInfo.identifier = "MY_" + moduleType.ToString () + "_MODULE_ID";

            evoModuleInfo.booleanParameters = CreateDummyParameters<bool> ("BooleanParameter", 1, false);
            evoModuleInfo.doubleParameters = CreateDummyParameters<double> ("DoubleParameter", 1, 3.14159265358979323846);
            evoModuleInfo.intParameters = CreateDummyParameters<int> ("IntegerParameter", 1, 42);
            evoModuleInfo.intSequenceParameters = CreateDummyParameters<int[]> ("IntSequenceParameter", 1,
                new int[] {1,1,2,3,5,8,13,21,34});
            evoModuleInfo.stringSequenceParameters = CreateDummyParameters<string[]> ("StringSequenceParameter", 1,
                new string[] {"blau","gelb","rot"});
            evoModuleInfo.stringParameters = CreateDummyParameters<string> ("StringParameter", 1,
                "In the beginning, the Universe was created. This has made a lot of people"
                + " very angry and been widely regarded as a bad move.");
            evoModuleInfo.operatorParameters = CreateDummyParameters<string[]> ("OperatorParameter", 1,
                new string[] {"Tick","Trick","Track"});
            evoModuleInfo.fileParameters = CreateDummyParameters<string> ("FileParameter", 1, "/dev/null");

            return evoModuleInfo;
        }

        public static SerializableDictionary<string, T> CreateDummyParameters<T>
            (string prefix, int count, T value)
        {
            SerializableDictionary<string, T> parameters = new SerializableDictionary<string, T> ();

            for (int i = 1; i <= count; ++i) {
                parameters.Add (prefix + "_" + i, value);
            }

            return parameters;
        }

        /// <summary>
        /// Erzeugt eine Liste von allen Assembly-Namen, die von Modulen in dieser Konfiguration
        /// verwendet werden. Jede Assembly kommt darin nur einmal vor.
        /// </summary>
        /// <returns>Eine Liste von Assembly-Namen.</returns>
        public List<string> GetAssemblyNames()
        {
            List<string> assemblyList = new List<string>();
            string tmpAssemblyName;

            if (initializationModule != null)
                assemblyList.Add(initializationModule.assemblyName);
            
            if (simulationModule != null)
            {
                tmpAssemblyName = simulationModule.assemblyName;

                if (!assemblyList.Contains(tmpAssemblyName))
                {
                    assemblyList.Add(tmpAssemblyName);
                }
            }

            if (moduleList != null)
            {
                foreach (EvoModuleInfo moduleInfo in moduleList)
                {
                    tmpAssemblyName = moduleInfo.assemblyName;
                    if (!assemblyList.Contains(tmpAssemblyName))
                    {
                        assemblyList.Add(tmpAssemblyName);
                    }
                }
            }

            return assemblyList;
        }

        /// <summary>
        /// Liest eine SimConfig aus der angegebenen Datei ein.
        /// </summary>
        /// <param name="filename">Die XML-Datei, in der sich die
        /// SimConfig befindet.</param>
        /// <exception cref="UnauthorizedAccessException">Der Aufrufer hat
        /// nicht die erforderliche Berechtigung.</exception>
        /// <exception cref="ArgumentException">filename ist kein gültiger
        /// Dateiname.</exception>
        /// <exception cref="ArgumentNullException">filename ist null.</exception>
        /// <exception cref="PathTooLongException">filename ist zu lange.</exception>
        /// <exception cref="DirectoryNotFoundException">Verzeichnis nicht
        /// gefunden.</exception>
        /// <exception cref="FileNotFoundException">filename wurde nicht gefunden.</exception>
        /// <returns>SimConfig-Objekt.</returns>
        /// <exception cref="NotSupportedException">Format von filename
        /// ist ungültig.</exception>
        /// <exception cref="OutOfMemoryException">Nicht genügend
        /// Speicherplatz für das Einlesen der Datei vorhanden.</exception>
        /// <exception cref="IOException">Ein Fehler tritt während dem
        /// Einlesen der Datei auf.</exception>
        /// <exception cref="Exception">Wenn beim Deserialisieren ein
        /// beliebiger Fehler auftritt.</exception>
        /// <returns>SimConfig-Objekt.</returns>
        public static SimConfig ReadFromFile (string filename)
        {
            return SimConfig.ReadFromStreamReader (System.IO.File.OpenText(filename));
        }

        /// <summary>
        /// Liest eine SimConfig aus dem angegebenen StreamReader ein und
        /// schließt den StreamReader anschließend.
        /// </summary>
        /// <exception cref="ArgumentNullException">Der angegebene
        /// StreamReader ist null.</exception>
        /// <param name="sr">Der StreamReader, der die XML-Datei enthält.</param>
        /// <exception cref="OutOfMemoryException">Nicht genügend
        /// Speicherplatz für das Einlesen der Datei vorhanden.</exception>
        /// <exception cref="IOException">Ein Fehler tritt während dem
        /// Einlesen der Datei auf.</exception>
        /// <exception cref="Exception">Wenn beim Deserialisieren ein
        /// beliebiger Fehler auftritt.</exception>
        /// <returns>SimConfig-Objekt.</returns>
        public static SimConfig ReadFromStreamReader (System.IO.StreamReader sr)
        {
            try
            {
                string simConfigString = sr.ReadToEnd();
                return (SimConfig) EvoLab.Common.Com.ComWrapper.ComDeserialize (
                    simConfigString, typeof(SimConfig));
            }
            finally
            {
                if (sr != null)
                    sr.Close ();
            }
        }
    }
}
