/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

namespace EvoLab.CoreApi.Individuals {

    [Serializable]
    public abstract class Vector<T, S>
        : IComparable<T>, ITransformable<T>, IFitnessReadable, ICloneable<T>
        where T : Vector<T, S>
    {
        public abstract T Clone ();

        // ~IsWorseThan
        public abstract int CompareTo (T other);

        protected double fitness;

        //TODO: make readonly and kick dpc (has to stay as long as stupid
        //xml-serialization is used)
        public S[] vector;

        protected internal Vector(int length)
        {
            vector = new S[length];
        }

        public double Fitness {
            get {
                return fitness;
            }
            set {
                fitness = value;
            }
        }

        public void UseValuesOf (T other)
        {
            this.fitness = other.fitness;
            Array.Copy (other.vector, this.vector, this.vector.Length);
        }

                public override string ToString() {
                        System.Text.StringBuilder sb = new System.Text.StringBuilder();
                        
                        for (int i = 0; i < vector.Length - 1; ++i) {
                                sb.Append(vector[i].ToString());
                                sb.Append(';');
                        }
                        sb.Append(vector[vector.Length - 1].ToString());
                        
                        return sb.ToString();
                }
                
    }

    [Serializable]
    public abstract class VectorFactory<T, S> : IndividualFactory<T> where T : Vector<T, S>
    {
        protected readonly int length;

        public VectorFactory (int length) {
            this.length = length;
        }

        public abstract T allocateNewIndividual ();
    }

    [Serializable]
    public class IntVector : Vector<IntVector, int>, ITransformable<IntVector> {

        public override int CompareTo (IntVector other)
        {
            return fitness.CompareTo (other.fitness);
        }

        public override IntVector Clone ()
        {
            IntVector clone = new IntVector (vector.Length);

            clone.fitness = fitness;
            Array.Copy (vector, clone.vector, vector.Length);

            return clone;
        }

        protected internal IntVector(int length)
            : base(length)
        {
            fitness = Double.MaxValue;
        }
    }

    [Serializable]
    public class IntVectorFactory : VectorFactory<IntVector, int>
    {
        public IntVectorFactory(int length) : base(length) { }

        public override IntVector allocateNewIndividual()
        {
            return new IntVector(length);
        }
    }

    [Serializable]
    public class FloatVector : Vector<FloatVector, float> {

        //TODO: make readonly and kick dpc (has to stay as long as stupid
        //xml-serialization is used)
        public FloatVector() : base (0) {} //kick ASAP

        public override int CompareTo (FloatVector other)
        {
            return fitness.CompareTo (other.fitness);
        }

        public override FloatVector Clone ()
        {
            FloatVector clone = new FloatVector (vector.Length);

            clone.fitness = fitness;
            Array.Copy (vector, clone.vector, vector.Length);

            return clone;
        }

        protected internal FloatVector (int length) : base (length)
        {
            fitness = Double.MaxValue;
        }
    }

    [Serializable]
    public class FloatVectorFactory : VectorFactory<FloatVector, float>
    {
        public FloatVectorFactory (int length) : base (length) {}

        public override FloatVector allocateNewIndividual ()
        {
            return new FloatVector (length);
        }
    }

    [Serializable]
    public class DoubleVector : Vector<DoubleVector, double> {

        //TODO: make readonly and kick dpc (has to stay as long as stupid
        //xml-serialization is used)
        public DoubleVector() : base (0) {} //kick ASAP

        public override int CompareTo (DoubleVector other)
        {
            return fitness.CompareTo (other.fitness);
        }

        public override DoubleVector Clone ()
        {
            DoubleVector clone = new DoubleVector (vector.Length);

            clone.fitness = fitness;
            Array.Copy (vector, clone.vector, vector.Length);

            return clone;
        }

        protected internal DoubleVector (int length) : base (length)
        {
            fitness = Double.MinValue;
        }
    }

    [Serializable]
    public class DoubleVectorFactory : VectorFactory<DoubleVector, double>
    {
        public DoubleVectorFactory (int length) : base (length) {}

        public override DoubleVector allocateNewIndividual ()
        {
            return new DoubleVector (length);
        }
    }
}
