/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

using EvoLab.CoreApi.Random;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Populations;
namespace EvoLab.CoreApi.Operators {

    public class Selection
    {
        public static void Elitist<T>
        (bool copyValues, T[] candidates, T[] selected, ProblemType problemType)
            where T : ITransformable<T>, IComparable<T>
        {
            Array.Sort (candidates);
            if (problemType == ProblemType.maximizeProblem)
                Array.Reverse(candidates);

            int j = 0;
            for (int i=0; i<selected.Length; ++i) {

                if (j == candidates.Length)
                    j = 0;

                if (copyValues)
                    selected[i].UseValuesOf (candidates[j]);
                else
                    selected[i] = candidates[j];

                j++;
            }
        }

        public static void LinearRankingIntuitive<T>
        (IRandom rand, bool copyValues, bool noDuplicates, T[] candidates, T[] selected, double[] probabilities, double rankInfluence, ProblemType problemType)
            where T : ITransformable<T>, IComparable<T>
        {
            CreateRankBasedProbabilities<T>(candidates, probabilities, rankInfluence, problemType);
            IntuitiveSelection<T> (rand, copyValues, noDuplicates, candidates, selected, probabilities);
        }

        public static void LinearRankingRouletteWheel<T>
        (IRandom rand, bool copyValues, T[] candidates, T[] selected, double[] probabilities, double rankInfluence, ProblemType problemType)
            where T : ITransformable<T>, IComparable<T>
        {
            CreateRankBasedProbabilities<T>(candidates, probabilities, rankInfluence, problemType);
            RouletteWheelSelection<T> (rand, copyValues, candidates, selected, probabilities);
        }

        public static void FitnessProportionalIntuitive<T>
        (IRandom rand, bool copyValues, bool noDuplicates, T[] candidates, T[] selected, double[] probabilities, ProblemType problemType)
            where T : ITransformable<T>, IFitnessReadable
        {
            AssignFitnessProportionalProbabilities<T>(candidates, probabilities, problemType);
            IntuitiveSelection<T> (rand, copyValues, noDuplicates, candidates, selected, probabilities);
        }

        public static void FitnessProportionalRouletteWheel<T>
        (IRandom rand, bool copyValues, T[] candidates, T[] selected, double[] probabilities, ProblemType problemType)
            where T : ITransformable<T>, IFitnessReadable
        {
            AssignFitnessProportionalProbabilities<T>(candidates, probabilities, problemType);
            RouletteWheelSelection<T> (rand, copyValues, candidates, selected, probabilities);
        }

        public static void AssignFitnessProportionalProbabilities<T>
        (T[] candidates, double[] probabilities, ProblemType problemType)
            where T : IFitnessReadable
        {
            if (candidates == null)
                throw new ArgumentNullException ("candidates");
            if (probabilities == null)
                throw new ArgumentNullException ("probabilities");

            double fitnesssum = 0.0d;
            foreach (T ind in candidates) {
                fitnesssum += ind.Fitness;
            }

            if (fitnesssum <= 0)
                throw new ArgumentException("Fitnessvalues of candidates are too similar. No fitnessproportional probabilities can be assigned.");

            for (int i=0; i<candidates.Length; ++i){
                if(problemType == ProblemType.maximizeProblem)
                    probabilities[i] = candidates[i].Fitness / fitnesssum;
                else
                    probabilities[i] = 1.0 - candidates[i].Fitness / fitnesssum;
            }                
        }

        public static void CreateRankBasedProbabilities<T>
        (T[] candidates, double[] probabilities, double rankInfluence, ProblemType problemType)
            where T : IComparable<T>
        {
            if (candidates == null)
                throw new ArgumentNullException ("candidates");
            if (probabilities == null)
                throw new ArgumentNullException ("probabilities");
            if ((rankInfluence < 1) || (rankInfluence > 2))
                throw new ArgumentOutOfRangeException ("rankInfluence muss im Intervall [1,2] liegen.");
            if (candidates.Length < 2)
                throw new ArgumentException ("Zu wenige Kandidaten für eine rangbasierte Selektion (mind. 2 erforderlich).");

            Array.Sort (candidates);
            if (problemType == ProblemType.maximizeProblem)
                Array.Reverse(candidates);

            double x = rankInfluence / (double) candidates.Length;
            double y = 2.0d * (x - 1.0d / (double) candidates.Length) / ((double) candidates.Length - 1.0d);

            for (int i=0; i<candidates.Length; ++i) {
                probabilities[i] = x - (double) i * y;
            }
        }

        public static void IntuitiveSelection<T>
        (IRandom rand, bool copyValues, bool noDuplicates, T[] candidates, T[] selected, double[] probabilities)
            where T : ITransformable<T>
        {
            if (rand == null)
                throw new ArgumentNullException ("rand");
            if (candidates == null)
                throw new ArgumentNullException ("candidates");
            if (selected == null)
                throw new ArgumentNullException ("selected");
            if (probabilities == null)
                throw new ArgumentNullException ("probabilities");
            if (noDuplicates && candidates.Length < selected.Length)
                throw new ArgumentException ("Wenn keine Duplikate verwendet werden dürfen muss die Auswahlmenge mindestens so groß sein wie die Zielmenge.");

            int availableCandidatesCount = candidates.Length;
            int i = 0;
            while (i < selected.Length) {
                int chosenIndex = rand.Next (availableCandidatesCount);

                if (rand.NextDouble () < probabilities[chosenIndex]) {
                    if (copyValues)
                        selected[i].UseValuesOf (candidates[chosenIndex]);
                    else
                        selected[i] = candidates[chosenIndex];

                    if (noDuplicates) {
                        availableCandidatesCount --;
                        T temp = candidates [chosenIndex];
                        candidates[chosenIndex] = candidates[availableCandidatesCount];
                        candidates[availableCandidatesCount] = temp;
                    }

                    ++i;
                }
            }
        }

        public static void RouletteWheelSelection<T>
        (IRandom rand, bool copyValues, T[] candidates, T[] selected, double[] probabilities)
            where T : ITransformable<T>
        {
            if (rand == null)
                throw new ArgumentNullException ("rand");
            if (candidates == null)
                throw new ArgumentNullException ("candidates");
            if (selected == null)
                throw new ArgumentNullException ("selected");
            if (probabilities == null)
                throw new ArgumentNullException ("probabilities");
            if (candidates.Length != probabilities.Length)
                throw new ArgumentOutOfRangeException ("candidates.Length muss gleich probabilities.Length sein.");

            //Transform probabilities into Roulettewheel
            double wheelPosition = 0.0d;
            int i;
            for (i=0; i<candidates.Length; ++i) {
                wheelPosition += probabilities[i];
                probabilities[i] = wheelPosition;
            }

            for (i=0; i<selected.Length; ++i) {

                double wheelAngle = rand.NextDouble () * wheelPosition;
                int index = Array.BinarySearch<double> (probabilities, wheelAngle);

                if (index < 0)
                    index = ~index;

                if (copyValues)
                    selected[i].UseValuesOf (candidates[index]);
                else
                    selected[i] = candidates[index];
            }
        }

        public static void Tournament<T>(IRandom rand, bool copyValues, bool noDuplicates, T[] participants, T[] winners, int n_competitors, ProblemType problemType)
            where T : IComparable<T>, ITransformable<T>
        {
            if (rand == null)
                throw new ArgumentNullException ("rand");
            if (participants == null)
                throw new ArgumentNullException ("participants");
            if (winners == null)
                throw new ArgumentNullException ("winners");
            if ((n_competitors < 2) || (n_competitors > participants.Length))
                throw new ArgumentOutOfRangeException ("n_competitors muss im Intervall [2,participants.Length] liegen.");
            if (noDuplicates && participants.Length < winners.Length)
                throw new ArgumentException ("Wenn keine Duplikate verwendet werden dürfen muss die Auswahlmenge mindestens so groß sein wie die Zielmenge.");

            T winner;
            int availableCandidatesCount = participants.Length;
            int randomIndex;
            int winnerIndex;
            int cmpVal = 1;

            if (problemType == ProblemType.maximizeProblem)
                cmpVal = -1;

            for (int i=0; i < winners.Length; i++) {
                randomIndex = rand.Next (availableCandidatesCount);
                winner = participants[randomIndex];
                winnerIndex = randomIndex;
                for (int j = 1; j < n_competitors; j++) {
                    randomIndex = rand.Next (availableCandidatesCount);
                    if (winner.CompareTo (participants[randomIndex]) == cmpVal) {
                        winner = participants[randomIndex];
                        winnerIndex = randomIndex;
                    }
                }
                if (copyValues)
                    winners[i].UseValuesOf (winner);
                else
                    winners[i] = winner;

                if (noDuplicates) {
                    availableCandidatesCount --;
                    participants[winnerIndex] = participants[availableCandidatesCount];
                    participants[availableCandidatesCount] = winner;
                }
            }
        }
    }
}
