/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.Collections.Generic;

using EvoLab.CoreApi;
using EvoLab.CoreApi.ModuleTemplates;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.Individuals;

namespace EvoLab.Core.StandardModules.Universal
{
        [Author("Lieferumfang von EvoLab")]
    [Description("Mutationsmodul, zyklisches Tauschen von n Stellen")]
        [Identifier("EA_Permutation_MutationShiftUpDown")]

    [IntParameter("MIN_N", 1)]
    [IntParameter("MAX_N", 1)]
    public class PermutationMutationShiftUpDown<I, P> : Mutation<I, P> where P : VectorPopulation<I, int> where I : Vector<I, int>
    {
        private int myMinN;
        private int myMaxN;

        private int vectorLength;

        public override void Init(string name, P population, IWorld world)
        {
            base.Init (name, population, world);

            vectorLength = population.VectorLength;

            myMinN = world.GetLocalInteger(name, "MIN_N");
            if ((myMinN < 1) ||(myMinN  > (vectorLength / 2)))
                throw new ArgumentOutOfRangeException("Parameter 'MIN_N' < 1 or > ('vectorLength' / 2).");

            myMaxN = world.GetLocalInteger(name, "MAX_N");
            if ((myMaxN < 1) || (myMaxN < myMinN) || (myMaxN > (vectorLength / 2)))
                throw new ArgumentOutOfRangeException("Parameter 'MAX_N' < 1 or < 'MIN_N' or > ('vectorLength' / 2).");

            if (myMaxN > ((vectorLength / 2) + 1))
                throw new ArgumentOutOfRangeException("Parameter 'MAX_N' > (('vectorLength' / 2) + 1).");
        }

        protected override void Mutate(I mutant)
        {
            int count = 0;
            if (myMinN == myMaxN)
                count = myMinN;
            else
                count = world.RandomNumberGenerator.Next(myMinN, myMaxN + 1);
           
            int mutatedPos = world.RandomNumberGenerator.Next(vectorLength);
            int firstValue = mutant.vector[mutatedPos];
            if (world.RandomNumberGenerator.NextDouble() < 0.5)
            {
                // Die 1. Stelle wird nach oben verschoben:
                mutant.vector[mutatedPos] += count;
                if (mutant.vector[mutatedPos] >= vectorLength)
                {
                    // Die Stelle �bersteigt nun die Dimension:
                    mutant.vector[mutatedPos] -= vectorLength;
                }
            }
            else
            {
                // Die 1. Stelle wird nach unten verschoben:
                mutant.vector[mutatedPos] -= count;
                if (mutant.vector[mutatedPos] < 0)
                {
                    // Die Stelle unterschreitet 0:
                    mutant.vector[mutatedPos] += vectorLength;
                }
            }

            // Nun wird die andere Stelle gesucht, mit welcher der Wert nun getauscht wird:
            for (int i = 0; i < vectorLength; i++)
            {
                if ((i != mutatedPos) && (mutant.vector[i] == mutant.vector[mutatedPos]))
                {
                    mutant.vector[i] = firstValue;
                    break;
                }
            }
        }
    }
}
