/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;
using System.Collections.Generic;

using EvoLab.CoreApi;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.ModuleTemplates;

namespace EvoLab.Core.StandardModules.Universal
{
        [Identifier("EA_Permutation_RecombinationEdges")]
        
    [IntParameter("PARENT_COUNT", 2)]

    public class PermutationRecombinationEdges<I, P> : EARecombination<I, P> where P : VectorPopulation<I, int> where I : Vector<I, int>
    {
        private int vectorLength;
        private int parentCount;

        // Die DIMENSION viele Adjazenzlisten, jeweils f�r eine Nummer:
        // Jede Zahl bekommt von jedem Eltern h�chstens 2 neue Nachbarn.
        // Die Adjazenzlisten sind hier als einzelnes 2D-Array gemacht (Perfomance).
        private int[,] adjacentMatrix;

        private bool[] evenUsed;

        public override void Init(
            string name, P population,
            IWorld world)
        {
            base.Init (name, population, world);

            vectorLength = population.VectorLength;

            parentCount = world.GetLocalInteger (name, "PARENT_COUNT");
            if (parentCount < 1 || parentCount > vectorLength)
                throw new ArgumentOutOfRangeException ("PARENT_COUNT must be in [1..tsp.length]");

            adjacentMatrix = new int[vectorLength, 2 * parentCount];

            InitNumbers (parentCount, 1);
        }

        protected override void Recombine (I[] parents, I[] children)
        {
            // Die Adjazenzlisten werden aufgebaut.
            int maxNeightbors = 2 * parentCount;  // Jeder Elter "f�gt" jeder zahl 2 Zahlen als Nachbarn zu. Maximal, weil es auch �berschneidungen geben kann.
            int aktNeightbor = 0;
            foreach (I par in parents)
            {
                for (int node = 1; node < vectorLength - 1; node++)
                {
                    adjacentMatrix[par.vector[node], aktNeightbor] = par.vector[node - 1]; // Der Vorg�nger im aktuellen Elter
                    adjacentMatrix[par.vector[node], aktNeightbor + 1] = par.vector[node + 1];  // Der NAchfolger im aktuellen Elter
                }
                // Der 1. und letzte Knoten m�ssen manuell bearbeitet werden:
                adjacentMatrix[par.vector[0], aktNeightbor] = par.vector[vectorLength - 1];
                adjacentMatrix[par.vector[0], aktNeightbor + 1] = par.vector[1];
                adjacentMatrix[par.vector[vectorLength - 1], aktNeightbor] = par.vector[vectorLength - 2];
                adjacentMatrix[par.vector[vectorLength - 1], aktNeightbor + 1] = par.vector[0];

                aktNeightbor += 2;
            }

            // Markiert alle bereits verwendeten Zahlen.
            // Doppelte Speicherung aus Eff-Gr�nden.
            evenUsed = new bool[vectorLength];
            List<int> notUsed = new List<int>();
            int remainingNodes = vectorLength - 2;
            for (int c = 0; c < vectorLength; c++)
            {
                notUsed.Add(c);
            }


            // Der 1. Wert ist der 1. Wert eines zuf�lligen Elters:
            int value = parents[world.RandomNumberGenerator.Next(parentCount)].vector[0];
            bool directNeightborFound;
            children[0].vector[0] = value;
            evenUsed[value] = true;
            notUsed.Remove(value);

            for (int node = 1; node < vectorLength; node++)
            {
                // Der n�chste Nachbar (der in einem Eltern existiert) wird gesucht:
                // Dabei wird eine Position in der Adjazenzliste per Zufall ermittelt,
                // falls diese Nummer bereits verbraucht ist wird Schritt f�r Schritt
                // die Liste weitergelaufen (damit nicht jedes mal eine Zufallz. gez. wird)
                directNeightborFound = false;
                int neightbor = world.RandomNumberGenerator.Next(maxNeightbors);
                int trys = 0;
                while (trys < maxNeightbors)
                {
                    if (!evenUsed[adjacentMatrix[value, neightbor]])
                    {
                        value = adjacentMatrix[value, neightbor];
                        directNeightborFound = true;
                        break;
                    }
                    neightbor++;
                    trys++;
                    if (neightbor == maxNeightbors) { neightbor = 0; };

                }

                // Es gibt keinen nocht nicht verwendeten Nachbarn mehr:
                if (!directNeightborFound)
                {
                    value = notUsed[world.RandomNumberGenerator.Next(remainingNodes)];
                }
                evenUsed[value] = true;
                notUsed.Remove(value);
                remainingNodes--;
                children[0].vector[node] = value;
            }
        }
    }
}
