/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

using EvoLab.CoreApi;
using EvoLab.CoreApi.ModuleTemplates;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.Individuals;

namespace EvoLab.Core.StandardModules.Universal
{
        [Identifier("EA_Permutation_RecombinationNCrossover")]
        
    [IntParameter("CROSSOVER_N", 2)]

    public class PermutationRecombinationNCrossover<I, P> : EARecombination<I, P> where P : VectorPopulation<I, int> where I : Vector<I, int>
    {
        private int vectorLength;
        private int n = 0;

        private bool[] firstChildUsedValues;
        private bool[] secondChildUsedValues;
        private bool[] crossOverPositions;

        public override void Init(string name, P population, IWorld world)
        {
            base.Init (name, population, world);

            vectorLength = population.VectorLength;

            n = world.GetLocalInteger(name, "CROSSOVER_N");
            if (n < 1 || n > vectorLength)
                throw new ArgumentOutOfRangeException("Parameter 'CROSSOVER_N' not in [1..vectorLength]");

            crossOverPositions = new bool[vectorLength];
            firstChildUsedValues = new bool[vectorLength];
            secondChildUsedValues = new bool[vectorLength];

            //Recombine now always passes us 2 Parents and 2 Children
            InitNumbers (2, 2);
        }

        private void ResetUsedValues ()
        {
            for (int i=0; i<vectorLength; ++i) {
                firstChildUsedValues[i] = false;
                secondChildUsedValues[i] = false;
                crossOverPositions[i] = false;
            }
        }

        //TODO: Refactor
        protected override void Recombine(I[] parents, I[] children)
        {
            ResetUsedValues ();

            I firstChildParent = parents[0];
            I secondChildParent = parents[1];
            I firstChild = children[0];
            I secondChild = children[1];

            for (int j = 0; j < n; j++) {
                int k = world.RandomNumberGenerator.Next(1, vectorLength);
                while (crossOverPositions[k])
                {
                    k = world.RandomNumberGenerator.Next(vectorLength);
                }
                crossOverPositions[k] = true;
            }

            for (int i=0; i<vectorLength; ++i) {

                if (crossOverPositions[i]) {
                    I temp = firstChildParent;
                    firstChildParent = secondChildParent;
                    secondChildParent = temp;
                }

                int value;
                int j;
                for (j=0; j<vectorLength; ++j) {
                    value = firstChildParent.vector[j];
                    if (!firstChildUsedValues[value])
                    {
                        firstChildUsedValues[value] = true;
                        firstChild.vector[i] = value;
                        break;
                    }
                }
                for (j=0; j<vectorLength; ++j) {
                    value = secondChildParent.vector[j];
                    if (!secondChildUsedValues[value])
                    {
                        secondChildUsedValues[value] = true;
                        secondChild.vector[i] = value;
                        break;
                    }
                }
            }
        }
    }
}
