/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

using EvoLab.CoreApi;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.ModuleTemplates;

namespace PopFree
{

    [Identifier("PopFree_AcceptorThreshold")]
    
    [DoubleParameter("TOLERANCE_PERCENTAGE", 0.05d)]
    [DoubleParameter("DECREASE_RATE", 0.99d)]

    public class PopFree_AcceptorThreshold<I, P> : Acceptor<I, P> where I : class, IFitnessReadable, IComparable<I>, ITransformable<I> where P : PopFreeVectorPopulation<I> 
    {
        private double tolerancePercentage;
        
        private double decreaseRate;
        
        public override void Init(String name, P population, IWorld world)
        {
            base.Init(name, population, world);

            // Vom Benutzer eingestellt Starttoleranz aus der Konfiguration holen
            tolerancePercentage =  world.GetLocalDouble(name, "TOLERANCE_PERCENTAGE");
            if ((tolerancePercentage < 0.0d) || (tolerancePercentage > 1.0d))
                throw new ArgumentOutOfRangeException("(tolerancePercentage < 0.0d) || (tolerancePercentage > 1.0d)");

            // Vom Benutzer eingestellt Vernderungsrate aus der Konfiguration holen
            decreaseRate = world.GetLocalDouble(name, "DECREASE_RATE");
            if ((decreaseRate < 0.0d) || (decreaseRate > 1.0d))
                throw new ArgumentOutOfRangeException("(decreaseRate < 0.0d) || (decreaseRate > 1.0d)");
        
        }

        protected override I Choose(I old, I challenger)
        {
            // Das zu bernehmende Individuum (vorerst das alte)
            I chosen = old;

            // Konstante fr den Vergleich zweier Individuuen
            // A.CompareTo(B) liefert 1 => A > B, -1 => A < B
            int cmpTo = 1;

            // Konstante entsprechend des Problemtyps setzen
            if (population.ProblemType == ProblemType.maximizeProblem)
                cmpTo = -1;

            // Ist das Neue (challenger) besser wie das alte?
            if (old.CompareTo(challenger) == cmpTo) {
                // Ja es ist besser -> bernehmen und merken "Besseres gefunden"
                population.BetterFound();
                // Das zurckgegebene Individuum wird als neues gesetzt (hier das neue)
                chosen = challenger;
            }
            else
            {
                //Nein, merken "Es wurde kein besseres gefunden"
                population.NothingBetterFound();
                
                //Quotient der beiden Fitnesswerte ermitteln -> Wieviel % Abweichung
                double quotient = (old.Fitness / challenger.Fitness);

                //Quotient soll < 1.0 sein (Vereinfachung)
                if (quotient > 1.0)
                    quotient = 1.0 / quotient;

                //Liegt die Abweichung in der aktuellen Toleranz?
                if (quotient < 1.0 - tolerancePercentage)
                {
                    //Nein, merken "Es wurde das schlechtere nicht angenommen"
                    population.WorseNotAccepted();
                } else {
                    //Ja, merken "Es wurde ein schlechteres angenommen"
                    chosen = challenger;
                    population.WorseAccepted();
                }
            }
            
            //Vernderung der Toleranzschwelle mittels der Vernderungsrate (sehr einfach!!)
            tolerancePercentage *= decreaseRate;

            //Ausgewhltes Individuum zurckgeben.
            return chosen;
        }
    }
}
