/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

using EvoLab.CoreApi;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.ModuleTemplates;

namespace EvoLab.Core.StandardModules.Universal
{

    [Identifier("PopFree_Permutation_GeneratorExchange")]
        
    [IntParameter("MIN_N", 2)]
    [IntParameter("MAX_N", 2)]

    public class PopFree_Permutation_GeneratorExchange<I, P> : Generator<I, P> where P : PopFreeVectorPopulation<I> where I : IntVector, ITransformable<I> 
    {
        protected int vectorLength;

        private int minN;
        private int maxN;

        private int[] positions;
        private int[] drawnPositions;        
        
        public override void Init(String name, P population, IWorld world, IModuleLoader<I,P> moduleLoader)
        {
            base.Init(name, population, world, moduleLoader);

            vectorLength = population.VectorLength;

            //Minimale Anzahl an Vertauschungen aus der Konfiguration auslesen
            minN = world.GetLocalInteger(name, "MIN_N");
            if ((minN < 2) || (minN > vectorLength ))
                throw new ArgumentOutOfRangeException("Parameter 'MIN_N' < 2 or > DIMENSION");

            //Maximale Anzahl an Vertauschungen aus der Konfiguration auslesen
            maxN = world.GetLocalInteger(name, "MAX_N");
            if ((maxN < 2) || (maxN < minN) || (maxN > vectorLength) )
                throw new ArgumentOutOfRangeException("Parameter 'MAX_N' < 2 or < 'MIN_N' or > vectorLength.");

            //Arrays anlegen (positions = Hilfsarray zur Ziehung der Zufallszahlen, 
            // drawnPositions = bereits gezogene Positionen)
            positions = new int[population.VectorLength];
            drawnPositions = new int [maxN];
        }

        protected override void GenerateNeighbour(I neighbour)
        {
            //Dieser Operator/Generator erzeugt ein Individuum aus der Nachbarschaft von I
            //hierbei ist die Nachbarschaft durch alle Individuuen gegeben, welche mittels
            //k-Tausch minN <= k <= maxN erreicht werden knnen

            int count = 0;

            //Wieviele Stellen sollen getauscht werden? (count = Anzahl der Vertauschungen)
            if (minN == maxN)
                count = minN;
            else
                count = world.RandomNumberGenerator.Next(minN, maxN + 1);

            //Array initialisieren
            for (int i=0; i<positions.Length; ++i) {
                positions[i] = i;
            }
            
            int nextPos;
            for (int i = 0; i < count; i++)
            {
                //Ziehen OHNE Zurcklegen, daher Pro Ziehung eine Weniger ziehen (-i)
                nextPos = world.RandomNumberGenerator.Next(0, vectorLength - i);
                //Bereits gezogene Zahlen merken
                drawnPositions[i] = nextPos;
                //Gezogene Zahl mit letzter im Array (der noch zu ziehenden (-i) berschreiben
                //5 Gezogen (vorher bereits vorher schon 4 gezogen)
                //1 2 3 10 5 6 7 8 9 10 (Array vor Vertauschung)
                //1 2 3 10 9 6 7 8 9 10 (Array nach Vertauschung)
                positions[nextPos] = vectorLength - i - 1;
            }
            //Gezogene Zahlen sortieren (laut Defintion im Skript i<j<k<...
            Array.Sort<int> (drawnPositions);            
            
            //Vertauschung vornehmen
            int lastValue = neighbour.vector[drawnPositions[count-1]];
            for (int i = count - 1; i > 0; i--)
            {
                neighbour.vector[drawnPositions[i]] = neighbour.vector[drawnPositions[i-1]];
            }            
            neighbour.vector[drawnPositions[0]] = lastValue;
        }
    }
}
