/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

using EvoLab.CoreApi;
using EvoLab.CoreApi.ModuleTemplates;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.Random;
using EvoLab.CoreApi.Operators;

namespace EvoLab.Core.StandardModules.Universal {

    [Identifier("Selection_Elitist")]
    public class ElitistSelection<I, P>
        : SelectionBase<I, P>
        where I : ITransformable<I>, IComparable<I>
        where P : IPopulation<I>, ITargetPopulation<I>
    {
        public override void Run ()
        {
            Selection.Elitist (true, population.Pop, population.TargetPop, population.ProblemType);
        }
    }

    [Identifier("Selection_Fitnessproportional")]
        
    public class FitnessProportionalRouletteWheelSelection<I, P>
        : SelectionBase<I, P>
        where P : IPopulation<I>, ITargetPopulation<I>
        where I : IFitnessReadable, ITransformable<I>
    {
        private IRandom rand;
        private double[] probabilities;

        public override void Init(string name, P population, IWorld world)
        {
            base.Init(name, population, world);
            rand = world.RandomNumberGenerator;
            probabilities = new double [0];
        }

        public override void Run()
        {
            I[] pop = population.Pop;
            if (pop.Length != probabilities.Length) {
                probabilities = new double [pop.Length];
            }
            Selection.FitnessProportionalRouletteWheel<I> (rand, true, pop,
                population.TargetPop, probabilities, population.ProblemType);
        }
    }

    [Identifier("Selection_Tournament")]

    [IntParameter("COMPETITOR_COUNT", 3)]
    [BooleanParameter ("NO_DUPLICATES", false)]
    public class TournamentSelection<I, P>
        : SelectionBase<I, P>
        where P : IPopulation<I>, ITargetPopulation<I>
        where I : IComparable<I>, ITransformable<I>
    {
        private int competitorCount;
        private IRandom rand;
        private bool noDuplicates;

        public override void Init (string name, P population, IWorld world)
        {
            base.Init(name, population, world);
                        
            competitorCount = world.GetLocalInteger(name, "COMPETITOR_COUNT");
            if (competitorCount < 2)
                throw new ArgumentOutOfRangeException("Parameter 'COMPETITOR_COUNT' muss mindestens 2 sein.");

            rand = world.RandomNumberGenerator;
            noDuplicates = world.GetLocalBool (name, "NO_DUPLICATES");
        }

        public override void Run ()
        {
            Selection.Tournament<I> (rand, true, noDuplicates, population.Pop, population.TargetPop, competitorCount, population.ProblemType);
        }
    }

    [Identifier("Selection_RankingBased")]

    [DoubleParameter("RANK_INFLUENCE", 1.5)]
    public class LinearRankingBasedRouletteWheelSelection<I, P>
        : SelectionBase<I, P>
        where P : IPopulation<I>, ITargetPopulation<I>
        where I : IComparable<I>, ITransformable<I>
    {
        private IRandom rand;
        private double [] probabilities;
        private double rankInfluence;

        public override void Init(
            string name, P population, IWorld world)
        {
            base.Init(name, population, world);
                        
            rand = world.RandomNumberGenerator;
            probabilities = new double [0];

            rankInfluence = world.GetLocalDouble (name, "RANK_INFLUENCE");
            if ((rankInfluence < 1) || (rankInfluence > 2))
                throw new ArgumentOutOfRangeException ("rankInfluence muss im Intervall [1,2] liegen.");
        }

        public override void Run ()
        {
            I[] pop = population.Pop;
            if (pop.Length != probabilities.Length) {
                probabilities = new double [pop.Length];
            }
            Selection.LinearRankingRouletteWheel<I> (rand, true, pop, population.TargetPop,
                probabilities, rankInfluence, population.ProblemType);
        }
    }

    [Identifier("Selection_Uniform")]

    public class UniformSelection<I, P>
        : SelectionBase<I, P>
        where I : ITransformable<I>
        where P : IPopulation<I>, ITargetPopulation<I>
    {
        public override void Run ()
        {
            for (int j=0; j<population.TargetPop.Length; ++j) {
                population.TargetPop[j].UseValuesOf
                    (population.Pop [world.RandomNumberGenerator.Next (population.Pop.Length)]);
            }
        }
    }
}
